<?php declare(strict_types=1);

namespace Infomaker\Everyware\Base;

use Infomaker\Everyware\Base\Interfaces\NewsItemPresentation;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\NewsMLTransformerManager;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;

/**
 * ArticleBodyParser
 *
 * @link    http://infomaker.se
 * @package Infomaker\Everyware\Base
 * @since   Infomaker\Everyware\Base\ArticleBodyParser 1.0.0
 */
class ArticleBodyParser
{

    /**
     * Contains a sortable map of all items
     *
     * @var array
     */
    private $item_map = [];

    /**
     * Contains a list of all valid body items
     *
     * @var array
     */
    private $body = [];

    public function __construct(PropertyObject $object, string $property_name = 'body_raw')
    {
        $transformer = NewsMLTransformerManager::createTransformer($property_name);
        $body_elements = $transformer->transform($object);

        foreach ($body_elements as $index => $body_element) {
            if ($body_element instanceof Item) {
                $this->addItem($body_element);
            }
        }
    }

    private function addItem(Item $item): void
    {
        $this->item_map[] = $item->getType();
        $this->body[] = $item;
    }

    public function getBody(): array
    {
        return $this->body;
    }

    public function hasType(string $type): bool
    {
        return \in_array($type, $this->item_map, true);
    }

    public function firstType()
    {
        return collect($this->item_map)->first();
    }

    public function firstOfType(string $type)
    {
        $all_of_type = $this->allOfType($type);

        return array_shift($all_of_type);
    }

    public function allPosOfType(string $type): array
    {
        return array_keys($this->item_map, $type);
    }

    public function removeType(string $type): void
    {
        foreach ($this->allPosOfType($type) as $key) {
            unset($this->item_map[$key], $this->body[$key]);
        }
    }

    public function allOfType(string $type): array
    {
        $items = [];

        foreach ($this->allPosOfType($type) as $key) {
            $items[] = $this->body[$key];
        }

        return $items;
    }

    public function allOfTypes(array $types): array
    {
        /* the inner empty array covers cases when no loops were made */
        $items = [[]];

        foreach ($types as $type) {
            $items[] = $this->allOfType($type);
        }

        return array_merge(...$items);
    }

    public function generateContent(NewsItemPresentation $presentation): array
    {
        $convertedBody = [];

        foreach ($this->getBody() as $index => $item) {
            if ( ! $item instanceof Item) {
                continue;
            }
            $convertedBody[] = $presentation->generateItem($item->getType(), $item);
        }

        return $convertedBody;
    }
}
