<?php declare(strict_types=1);

namespace Infomaker\Everyware\Base;

use Infomaker\Everyware\Support\Arr;
use Infomaker\Everyware\Support\Str;

class FeedRouter
{
    /**
     * The file where the feed-routes are registered and handled
     *
     * @var string
     */
    protected static $feedFile = 'feed.php';

    /**
     * Contains the registered types of feed templates
     *
     * @var array
     */
    protected static $registeredRoutes = [];

    /**
     * Contains the name of the GET-parameter where the route is stored
     *
     * @var string
     */
    protected static $routeParameter = 'route';

    /**
     * Contains all supported types of feed
     *
     * @var array
     */
    protected static $validFeeds = ['rss', 'rss2', 'atom', 'rdf'];

    /**
     * Singleton container
     *
     * @var FeedRouter
     */
    private static $instance;

    /**
     * Contains the type of the requested feed
     *
     * @var string Type of feed.
     */
    protected $feedType;

    /**
     * Contains the requested route
     *
     * @var string
     */
    protected $route;

    /**
     * Constructor
     */
    private function __construct()
    {
    }

    /**
     * Load the feed-template file and route to a registered route
     *
     * @return void
     */
    public function loadFeedFile(): void
    {
        locate_template(static::$feedFile, true);
        $route = '/' . trim(array_get($_GET, static::$routeParameter, ''), '/');
        $feedType = array_get($_GET, 'feed', 'rss');
        $this->routeRequest($route, $feedType);
    }

    /**
     * Add a route to match with
     *
     * @param $route
     * @param $callback
     *
     * @return void
     */
    public function addRoute($route, $callback): void
    {
        static::$registeredRoutes['/' . trim($route, '/')] = $callback;
    }

    /**
     * Match route with registered patterns
     *
     * @param string $route
     *
     * @return mixed
     */
    public function getMatchedPattern(string $route)
    {

        // Check if no exact match exist
        if ( ! array_key_exists($route, static::$registeredRoutes)) {

            foreach (array_keys(static::$registeredRoutes) as $registeredRoute) {

                if ($this->matchesRoutePattern($route, $registeredRoute)) {
                    return $registeredRoute;
                }

                if (Str::is($registeredRoute, $route)) {
                    return $registeredRoute;
                }
            }
        }

        return $route;
    }

    /**
     * Check for match among routs with optional parameters or
     * check for match with wildcard route
     *
     * @param string $pattern
     * @param string $route
     *
     * @return bool
     */
    protected function matchesRoutePattern(string $route, string $pattern): bool
    {
        return (bool)preg_match($this->createRegExpPattern($pattern), $route) || Str::is($pattern, $route);
    }

    /**
     * Create a regular expression to match routes with optional parameters
     *
     * @param string $routePattern
     *
     * @return string
     */
    protected function createRegExpPattern(string $routePattern): string
    {
        return '/^' . str_replace('/', '\/', preg_replace('/\{([\w-]+?)\}/', '([\w-]+?)', $routePattern)) . '$/';
    }

    /**
     * Get the values of the optional parameters from the route.
     *
     * @param $route
     * @param $pattern
     *
     * @return array
     */
    protected function extractOptionalParameters(string $route, string $pattern): array
    {
        if ($pattern === '/*') {
            return explode('/', trim($route, '/'));
        }

        preg_match_all($this->createRegExpPattern($pattern), $route, $matches);

        return isset($matches[1]) ? Arr::flatten(array_slice($matches, 1)) : [];
    }

    /**
     * Route the requested feed by finding a match from the list of registered route-patterns
     * and sending the optional parameters
     *
     * @param string $route
     * @param string $feedType
     *
     * @return void
     */
    protected function routeRequest(string $route, string $feedType = 'rss'): void
    {
        $routeParts = explode('/', trim($route, '/'));

        // Decode and slugify every parameter before proceeding
        $decodedRoute = '/' . implode('/', array_map(function ($part) {
                return Str::slug(urldecode($part));
            }, $routeParts));

        $matchedPattern = $this->getMatchedPattern($decodedRoute);
        $params = $this->extractOptionalParameters($decodedRoute, $matchedPattern);

        // Filter original routeParts to only keep the requested params
        $filteredParams = array_filter($routeParts, function ($part) use ($params) {
            return in_array(Str::slug(urldecode($part)), $params, true);
        });

        $this->callRoute($matchedPattern, $feedType, $filteredParams);
    }

    /**
     * Call registered function with matching parameters
     *
     * @param string $route
     * @param string $type
     * @param array  $params
     *
     * @return void
     */
    protected function callRoute(string $route, string $type, array $params = []): void
    {
        if (array_key_exists($route, static::$registeredRoutes)) {
            header('Content-Type: ' . feed_content_type($type) . '; charset=' . get_option('blog_charset'), true, 200);
            call_user_func_array(static::$registeredRoutes[$route], $params);
        }
    }

    public static function getInstance(): self
    {
        if (static::$instance === null) {
            static::$instance = new static;
        }

        return static::$instance;
    }

    public static function init(): FeedRouter
    {
        $router = static::getInstance();
        foreach (static::$validFeeds as $feed) {
            // Remove WP-action
            remove_all_actions("do_feed_{$feed}");
            // Route to feed-file
            add_action("do_feed_{$feed}", [$router, 'loadFeedFile'], 10, 1);
        }

        /** Change Rss url */
        remove_action('wp_head', 'feed_links_extra', 3); // Displays the links to the extra feeds such as category feeds
        remove_action('wp_head', 'feed_links', 2); // Displays the links to the general feeds: Post and Comment Feed

        // Add route as query var to wordpress
        \add_filter('query_vars', function (array $queryVars = []): array {
            $queryVars[] = 'route';

            return $queryVars;
        });
        // Route feeds
        \add_filter('request', function (array $queryVars = []): array {
            return isset($queryVars['route']) ? $_GET : $queryVars;
        });

        return $router;
    }

    /**
     * Helper-function to add a route to match for
     *
     * @param string   $route
     * @param callable $callback
     *
     * @return void
     */
    public static function register(string $route, callable $callback): void
    {
        static::getInstance()->addRoute($route, $callback);
    }
}
