<?php declare(strict_types=1);

namespace Infomaker\Everyware\Base\OpenContent;

use Infomaker\Everyware\Base\ContentProvider;
use Infomaker\Everyware\Base\PropertyMapConfig;
use Infomaker\Everyware\Base\PropertyParser;
use Infomaker\Everyware\Base\PropertyMapper;
use OcAPI;

class OpenContentProvider implements ContentProvider
{

    /**
     * @var PropertyMapper
     */
    protected static $property_mapper;

    /**
     * @var PropertyParser
     */
    protected static $property_parser;

    /**
     * @var string
     */
    private $content_reference;

    /**
     * @var OcAPI
     */
    private $oc;

    /**
     * @var string
     */
    private $query_params;

    /**
     * @var bool
     */
    private $use_cache;

    public function __construct(OcAPI $oc, array $query_params = [], $use_cache = true)
    {
        $this->oc = $oc;
        $this->query_params = $query_params;
        $this->use_cache = $use_cache;
    }

    public static function setupPropertyMapper(PropertyMapConfig $config): void
    {
        static::$property_mapper = new PropertyMapper($config);
        static::$property_parser = new PropertyParser($config);
    }

    public static function setupDefaultSearchProperties(array $properties = []): void
    {
        add_filter('ew_apply_default_search_properties', function ($filtered_properties) use ($properties) {
            return array_merge($filtered_properties, $properties);
        });
    }

    public static function setupNotifierProperties($contenttype, array $properties = []): void
    {
        $contenttype_var = strtolower($contenttype);

        add_filter("ew_notifier_update_{$contenttype_var}_properties",
            function ($filtered_properties) use ($properties) {
                return array_merge($filtered_properties, $properties);
            });
    }

    /**
     * Setup required search properties
     *
     * TODO: Refactored to do more generic and no hardcoded values
     */
    public static function setupRequiredSearchProperties(): void
    {
        add_filter('ew_apply_required_search_properties', function ($properties) {
            if (\in_array('Articles.uuid', $properties, true) && ! \in_array('Articles.Pubdate', $properties, true)) {
                $properties[] = 'Articles.Pubdate';
            }

            if (in_array('Articles.uuid', $properties, true) && ! in_array('Articles.created', $properties, true)) {
                $properties[] = 'Articles.created';
            }

            if (in_array('Articles.Articles.uuid', $properties, true) && ! in_array('Articles.Articles.Pubdate',
                    $properties, true)) {
                $properties[] = 'Articles.Articles.Pubdate';
            }


            if (in_array('Articles.Articles.uuid', $properties, true) && ! in_array('Articles.Articles.created',
                    $properties, true)) {
                $properties[] = 'Articles.Articles.created';
            }

            return $properties;
        });
    }

    public function queryWithRequirements(array $requirements = []): array
    {
        if ($this->content_reference) {
            $requirements['properties'] = $this->mergeMappedProperties($this->content_reference,
                $requirements['properties'] ?? []);

            return $this->mapResult($this->getContent($requirements));
        }

        return $this->getContent($requirements);
    }


    public function setPropertyMap(string $reference): void
    {
        $this->content_reference = $reference;
    }

    private function mapResult(array $result = []): array
    {
        return static::$property_parser->fromSearch($this->content_reference, $result)->all();
    }

    private function cleanResult(array $result = []): array
    {
        unset($result['facet'], $result['hits'], $result['duration']);

        return $result;
    }

    public static function setup(array $query_params = [], $use_cache = true)
    {
        return new static(new OcAPI(), $query_params, $use_cache);
    }

    public function mergeMappedProperties($content_reference, array $properties = []): array
    {
        return array_unique(array_merge($properties, $this->getPropertiesByReference($content_reference)));
    }

    private function getContent(array $requirements): array
    {
        return $this->cleanResult($this->oc->search(array_replace($this->query_params, $requirements),
            $this->use_cache));
    }

    public function getPropertiesByReference(string $reference = ''): array
    {
        return static::$property_mapper->requiredProperties($reference);
    }

    public function getObject(string $uuid, string $property_reference = '')
    {
        if ($this->content_reference) {
            $result = $this->oc->get_single_object(
                $uuid,
                $this->getPropertiesByReference($this->content_reference)
            );

            return static::$property_parser->fromOcObject($this->content_reference, $result);
        }

        return $this->oc->get_single_object(
            $uuid,
            $this->getPropertiesByReference($property_reference)
        );
    }
}
